@@ -124,7 +124,14 @@ def get_neighbours_from_tris(tris, label=None):
             neighbours[k] = set(neighbours[k]).intersection(label)
         else :
             neighbours[k]=f7(neighbours[k])
-    return np.array(neighbours)
+    return np.array(neighbours,dtype=object)
+
+def mask_colours(colours,triangles,mask):
+    """grey out mask"""
+    if mask is not None:
+        verts_masked = mask[triangles].any(axis=1)
+        colours[verts_masked,:] = np.array([0.86,0.86,0.86,1])
+    return colours
 
 def adjust_colours_pvals(colours, pvals,triangles,mask=None):
     """red ring around clusters and greying out non-significant vertices"""
@@ -151,8 +158,12 @@ def add_parcelation_colours(colours,parcel,triangles,labels=None,mask=None):
         colours[verts_masked,:] = np.array([0.86,0.86,0.86,1])    
     #normalise rois and colors
     rois=list(set(parcel))
-    if labels == None : 
+    if 0 in rois:
+        rois.remove(0)
+    if labels is  None : 
         labels = dict(zip(rois, np.random.rand(len(rois),4)))
+    
+    #remove transparent rois
     #find vertices that delineate rois
     neighbours=get_neighbours_from_tris(triangles)
     matrix_colored = np.zeros([len(triangles), len(rois)])
@@ -161,9 +172,9 @@ def add_parcelation_colours(colours,parcel,triangles,labels=None,mask=None):
         if len(ring)>0:
             ring_label = np.zeros(len(neighbours)).astype(bool)
             ring_label[ring]=1
-            ring=get_ring_of_neighbours(ring_label,neighbours)
-            ring_label[ring]=1
-            matrix_colored[:,l] = ring_label[triangles].sum(axis=1)
+#            ring=get_ring_of_neighbours(ring_label,neighbours)
+#            ring_label[ring]=1
+            matrix_colored[:,l] = np.median(ring_label[triangles],axis=1) #ring_label[triangles].sum(axis=1)
     #update colours with delineation
     maxis = [max(matrix_colored[i,:]) for i in range(0,len(colours))]
     colours = np.array([labels[rois[np.random.choice(np.where(matrix_colored[i,:] == maxi)[0])]] 
@@ -201,17 +212,65 @@ def normalized(a, axis=-1, order=2):
     return a / np.expand_dims(l2, axis)
 
 
-def plot_surf(vertices, faces,overlay,rotate=[270,90], cmap='viridis', filename='plot.png', label=False,
-             vmax=None, vmin=None, x_rotate=270, pvals=None, colorbar=True,
+def plot_surf(vertices, faces,overlay, rotate=[90,270], cmap='viridis', filename='plot.png', label=False,
+             vmax=None, vmin=None, x_rotate=270, pvals=None, colorbar=True, cmap_label='value',
              title=None, mask=None, base_size=6, arrows=None,arrow_subset=None,arrow_size=0.5,
-            alpha_colour = None,flat_map=False, z_rotate=0,cmap_label='value',parcel=None, parcel_cmap=None):
-    """plot mesh surface with a given overlay
-    vertices - vertex locations
-    faces - triangles of vertex indices definings faces
-    overlay - array to be plotted
-    cmap - matplotlib colormap
-    cmap_label - label for the colormap
-    rotate - 270 for lateral on lh, 90 for medial
+              arrow_colours = None,arrow_head=0.05,arrow_width=0.001,
+            alpha_colour = None,flat_map=False, z_rotate=0,parcel=None, parcel_cmap=None,):
+    """ This function plot mesh surface with a given overlay. 
+        Features available : display in flat surface, display parcellation on top, display gradients arrows on top
+
+    
+    Parameters:
+    ----------
+        vertices     : numpy array  
+                       vertex locations
+        faces        : numpy array
+                       triangles of vertex indices definings faces
+        overlay      : numpy array
+                       array to be plotted
+        rotate       : tuple, optional
+                       rotation angle for lateral on lh,  and medial 
+        cmap         : string, optional
+                       matplotlib colormap
+        filename     : string, optional
+                       name of the figure to save
+        label        : bool, optional
+                       colours smoothed (mean) or median if label         
+        vmin, vmax   : float, optional
+                       min and max value for display intensity
+        x_rotate     : int, optional
+        
+        pvals        : bool, optional
+        
+        colorbar     : bool, optional
+                       display or not colorbar
+        cmap_label   : string, optional
+                       label of the colorbar 
+        title        : string, optional
+                       title of the figure
+        mask         : numpy array, optional
+                       vector to mask part of the surface
+        base_size    : int, optional
+        
+        arrows       : numpy array, optional
+                       dipsplay arrows in the directions of gradients on top of the surface
+        arrow_subset : numpy array, optional
+                       vector containing at which vertices display an arrow
+        arrow_size   : float, optional
+                       size of the arrow
+        arrow_colours: 
+        alpha_colour : float, optional
+                       value to play with transparency of the overlay
+        flat_map     : bool, optional
+                       display on flat map 
+        z_rotate     : int, optional
+        
+        parcel       : numpy array, optional
+                       delineate rois on top of the surface
+        parcel_cmap  : dictionary, optional
+                       dic containing labels and colors associated for the parcellation
+                         
     """
     vertices=vertices.astype(np.float)
     F=faces.astype(int)
@@ -227,7 +286,6 @@ def plot_surf(vertices, faces,overlay,rotate=[270,90], cmap='viridis', filename=
         rotate=[90]
         intensity = np.ones(len(F))
     else:
-
         #change light source if z is rotate
         light = np.array([0,0,1,1]) @ yrotate(z_rotate)
         intensity=shading_intensity(vertices, F, light=light[:3],shading=0.7)
@@ -247,39 +305,42 @@ def plot_surf(vertices, faces,overlay,rotate=[270,90], cmap='viridis', filename=
         if vmax is not None:
             colours = (colours - vmin)/(vmax-vmin)
             colours = np.clip(colours,0,1)
-        else:
-            colours = (colours - colours.min())/(colours.max()-colours.min())
+        else: 
             vmax = colours.max()
             vmin = colours.min()
-        C = plt.get_cmap(cmap)(colours) 
+            colours = (colours - colours.min())/(colours.max()-colours.min())
+        C = plt.get_cmap(cmap)(colours)
         if alpha_colour is not None:
             C = adjust_colours_alpha(C,np.mean(alpha_colour[F],axis=1))
         if pvals is not None:
             C = adjust_colours_pvals(C,pvals,F,mask)
+        elif mask is not None:
+            C = mask_colours(C,F,mask)
         if parcel is not None :
             C = add_parcelation_colours(C,parcel,F,parcel_cmap,mask)
-        
             
         #adjust intensity based on light source here
-
         C[:,0] *= intensity
         C[:,1] *= intensity
         C[:,2] *= intensity
+        
+        collection = PolyCollection([], closed=True, linewidth=0,antialiased=False, facecolor=C, cmap=cmap)
         for i,view in enumerate(rotate):
             MVP = perspective(25,1,1,100)  @ translate(0,0,-3) @ yrotate(view) @ zrotate(z_rotate)  @ xrotate(x_rotate) @ zrotate(270*flat_map)
             #translate coordinates based on viewing position
             V = np.c_[vertices, np.ones(len(vertices))]  @ MVP.T
             
             V /= V[:,3].reshape(-1,1)
-
+            center = np.array([0, 0, 0, 1]) @ MVP.T;
+            center /= center[3];
             # add vertex positions to A_dir before transforming them
             if arrows is not None: 
-                #TODO consider adding small extra shift in
-                #surface normal direction to shift arrows out a bit
+                #calculate arrow position + small shift in surface normal direction
                 vertex_normal_orig = vertex_normals(vertices,faces)
-                
                 A_base = np.c_[vertices+vertex_normal_orig*0.01, np.ones(len(vertices))]  @ MVP.T
                 A_base /= A_base[:,3].reshape(-1,1)
+                
+                #calculate arrow direction
                 A_dir = np.copy(arrows) 
                 #normalise arrow size
                 max_arrow = np.max(np.linalg.norm(arrows,axis=1))
@@ -303,29 +364,43 @@ def plot_surf(vertices, faces,overlay,rotate=[270,90], cmap='viridis', filename=
             T, s_C = T[I,:], s_C[I,:]
             ax = fig.add_subplot(len(overlays),len(rotate)+1,2*k+i+1, xlim=[-.98,+.98], ylim=[-.98,+.98],aspect=1, frameon=False,
              xticks=[], yticks=[])
-            collection = PolyCollection(T, closed=True, linewidth=0,antialiased=False, facecolor=s_C)
+            collection = PolyCollection(T, closed=True, linewidth=0,antialiased=False, facecolor=s_C, cmap=cmap)
             collection.set_alpha(1)
             ax.add_collection(collection)
-            
+            #add arrows to image
             if arrows is not None:
                 front_arrows = F[front].ravel()
-                for i, arrow in enumerate(A_dir):
-                    #TODO find way to filter out arrows that should be hidden
-                    if i in arrow_subset and i in front_arrows: # and A_base[i,2]-3:
-                        half = A_dir[i,[0,1]] * 0.5
+                for arrow_index,i in enumerate(arrow_subset):
+                    if i in front_arrows and A_base[i,2] < center[2] + 0.01:
+                        arrow_colour = 'k'
+                        if arrow_colours is not None:
+                            arrow_colour = arrow_colours[arrow_index]
+                        #if length of arrows corresponds perfectly with coordinates
+                        # assume 1:1 matching
+                        if len(A_dir) == len(A_base):
+                            direction = A_dir[i]
+                        #otherwise, assume it is a custom list matching the 
+                        elif len(A_dir) == len(arrow_subset):
+                            direction  = A_dir[arrow_index]
+                        half = direction * 0.5
+                        
                         ax.arrow(A_base[i,0] - half[0],
                                  A_base[i,1] - half[1], 
-                                 A_dir[i,0], A_dir[i,1], 
-                                 head_width=0.01)
+                                 direction[0], direction[1], 
+                                 head_width=arrow_head,width =arrow_width,
+                                color = arrow_colour)
                     # ax.arrow(A_base[idx,0], A_base[idx,1], A_dir[i,0], A_dir[i,1], head_width=0.01)
             plt.subplots_adjust(left =0 , right =1, top=1, bottom=0,wspace=0, hspace=0)
+        
     if colorbar:
         l=0.7
         if len(rotate)==1:
             l=0.5
         cbar_size= [l, 0.3, 0.03, 0.38]
-        cbar = fig.colorbar(cm.ScalarMappable( cmap=cmap),
-                            ticks=[0,0.5, 1],cax = fig.add_axes(cbar_size))
+        cbar = fig.colorbar(collection,
+                            ticks=[0,0.5, 1],
+                            cax = fig.add_axes(cbar_size),
+                           )
         cbar.ax.set_yticklabels([np.round(vmin,decimals=2), np.round(np.mean([vmin,vmax]),decimals=2),
                          np.round(vmax,decimals=2)])
         cbar.ax.tick_params(labelsize=25)
